<?php
/**
 * --------------------------------------------------------------------------------
 *  Payment - Stripe checkout
 * --------------------------------------------------------------------------------
 * @package     Joomla 3.x
 * @subpackage  J2 Store
 * @author      Alagesan, J2Store <support@j2store.org>
 * @copyright   Copyright (c) 2019 J2Store . All rights reserved.
 * @license     GNU GPL v3 or later
 * @link        https://www.j2store.org
 * --------------------------------------------------------------------------------
 *
 * */
defined('_JEXEC') or die;
require_once(JPATH_ADMINISTRATOR . '/components/com_j2store/library/plugins/payment.php');

class plgJ2StorePayment_stripecheckout extends J2StorePaymentPlugin
{
    /**
     * @var $_element  string  Should always correspond with the plugin's filename,
     *                         forcing it to be unique
     */
    var $_element = 'payment_stripecheckout';
    var $_isLog = false;
    var $_j2version = null;

    /**
     * Constructor
     *
     * For php4 compatability we must not use the __constructor as a constructor for plugins
     * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
     * This causes problems with cross-referencing necessary for the observer design pattern.
     *
     * @param object $subject The object to observe
     * @param array $config An array that holds the plugin configuration
     * @since 1.5
     */
    function __construct(& $subject, $config)
    {
        parent::__construct($subject, $config);
        $this->loadLanguage('', JPATH_ADMINISTRATOR);
        if ($this->params->get('debug', 0)) {
            $this->_isLog = true;
        }
        if ($this->params->get('sandbox', 0)) {
            // get sandbox credentials
            $this->secret_key = trim($this->params->get('stripe_test_secret_key',''));
            $this->publish_key = trim($this->params->get('stripe_test_publish_key',''));
        } else {
            $this->secret_key = trim($this->params->get('stripe_secret_key',''));
            $this->publish_key = trim($this->params->get('stripe_publish_key',''));
        }
        F0FModel::addIncludePath(JPATH_SITE.'/plugins/j2store/'.$this->_element.'/'.$this->_element.'/models');
    }

    /**
     * verify Accept Subscription payment before payment
     * */
    function onJ2StoreAcceptSubscriptionPayment($element)
    {
        if ($this->_isMe($element)) {
            return true;
        } else {
            return null;
        }
    }

    /**
     * verify Accept Subscription payment with trial before payment
     * */
    function onJ2StoreAcceptSubscriptionPaymentWithTrial($element)
    {
        if ($this->_isMe($element)) {
            return true;
        } else {
            return null;
        }
    }

    /**
     * verify Accept Subscription card update
     * */
    function onJ2StoreAcceptSubscriptionCardUpdate($element)
    {
        if ($this->_isMe($element)) {
            return true;
        } else {
            return null;
        }
    }

    /**
     * To process renewal payment
     * */
    function onJ2StoreProcessRenewalPayment($paymentType, $subscription, $order)
    {
        if ($paymentType == $this->_element) {
            $this->byRefference($subscription, $order);
        }
    }

    function onJ2StoreAfterDisplayOrder($order){
        $html = '';
        if($order->orderpayment_type == $this->_element){
            $html = $this->loadCompletePaymentButton($order);
        }

        return $html;
    }

    public function onJ2StoreDisplayAdditionalContentAfterEachRelatedOrderStatusInSubscription($order){
        $html = '';
        if($order->orderpayment_type == $this->_element){
            $html = $this->loadCompletePaymentButton($order);
        }

        return $html;
    }

    public function onJ2StoreDisplayAdditionalContentAfterParentOrderStatusInSubscription($order){
        $html = '';
        if($order->orderpayment_type == $this->_element){
            $html = $this->loadCompletePaymentButton($order);
        }

        return $html;
    }

    protected function loadCompletePaymentButton($order){
        $html = '';
        $model = F0FModel::getTmpInstance('AppPayment_stripecheckout', 'J2StoreModel');
        $need_authentication = $model->checkEligibleForPaymentAuthentication($order, $this->params);
        if($need_authentication){
            $vars = new JObject();
            $vars->version = $this->_j2version;
            $vars->params = $this->params;
            $vars->order = $order;
            $vars->app_id = JPluginHelper::getPlugin('j2store', $this->_element)->id;
            $html = $this->_getLayout('authenticate_card_button', $vars);
        }

        return $html;
    }

    /**
     * Check subscription product available for an order
     * */
    protected function checkHasSubscriptionProductFromOrderID($order_id)
    {
        $j2StorePlugin = J2Store::plugin();
        $hasSubProduct = 0;
        $j2StorePlugin->event('CheckHasSubscriptionProductFromOrderID', array('app_subscriptionproduct', $order_id, &$hasSubProduct));
        return $hasSubProduct;
    }

    function onJ2StoreBeforeCheckout($order)
    {
        $document = JFactory::getDocument();
        $document->addScript('https://js.stripe.com/v3/');
    }

    public function setStripAppVersion()
    {
        require_once JPATH_SITE . "/plugins/j2store/" . $this->_element . "/library/vendor/autoload.php";
        // initialise stripe object
        \Stripe\Stripe::setApiKey($this->secret_key);
        \Stripe\Stripe::setAppInfo("Stripe Checkout", "1.0.0", "https://www.j2store.org");
    }

    function onJ2StoreCalculateFees($order)
    {
        //is customer selected this method for payment ? If yes, apply the fees
        $payment_method = $order->get_payment_method();

        if ($payment_method == $this->_element) {
            $total = $order->order_subtotal + $order->order_shipping + $order->order_shipping_tax;
            $surcharge = 0;
            $surcharge_percent = $this->params->get('surcharge_percent', 0);
            $surcharge_fixed = $this->params->get('surcharge_fixed', 0);
            if ((float)$surcharge_percent > 0 || (float)$surcharge_fixed > 0) {
                //percentage
                if ((float)$surcharge_percent > 0) {
                    $surcharge += ($total * (float)$surcharge_percent) / 100;
                }

                if ((float)$surcharge_fixed > 0) {
                    $surcharge += (float)$surcharge_fixed;
                }

                $name = $this->params->get('surcharge_name', JText::_('J2STORE_CART_SURCHARGE'));
                $tax_class_id = $this->params->get('surcharge_tax_class_id', '');
                $taxable = false;
                if ($tax_class_id && $tax_class_id > 0) $taxable = true;
                if ($surcharge > 0) {
                    $order->add_fee($name, round($surcharge, 2), $taxable, $tax_class_id);
                }
            }
        }
    }

    /**
     * Prepares variables and
     * Renders the form for collecting payment info
     *
     * @return string
     */
    function _renderForm($data)
    {
        $vars = new JObject ();
        $vars->prepop = array();
        $vars->onselection_text = $this->params->get('onselection', '');
        $html = $this->_getLayout('form', $vars);
        return $html;
    }

    /**
     * To load subscription card update form
     * */
    function onJ2StoreLoadSubscriptionPaymentCardUpdateForm($subscription, $order, $app_id)
    {
        if ($subscription->payment_method == $this->_element) {
            $document = JFactory::getDocument();
            $script = '(function($) {';
            $script .= ' $(document).ready(function(){';
            $script .=      '$("#button-subscription_update_card").hide();';
            $script .=      '$("#button-subscription_update_card").trigger("click");';
            $script .= ' });';
            $script .= '})(j2store.jQuery);';
            $document->addScriptDeclaration($script);
            $document->addScript('https://js.stripe.com/v3/');

            $vars = new JObject ();
            $vars->prepop = array();
            $vars->onselection_text = $this->params->get('onselection', '');
            $html = $this->_getLayout('form', $vars);
            return $html;
        }
    }

    /**
     * @param $data     array       form post data
     * @return string   HTML to display
     */
    function _prePayment($data)
    {
        $vars = new stdClass();
        $vars->order_id = $data ['order_id'];
        $vars->orderpayment_id = $data ['orderpayment_id'];
        $vars->orderpayment_amount = $data ['orderpayment_amount'];
        $vars->orderpayment_type = $this->_element;

        $vars->display_name = $this->params->get('display_name', 'PLG_J2STORE_PAYMENT_STRIPECHECKOUT');
        $vars->onbeforepayment_text = $this->params->get('onbeforepayment', '');
        $vars->button_text = $this->params->get('button_text', 'J2STORE_PLACE_ORDER');

        $order = F0FTable::getInstance('Order', 'J2StoreTable')->getClone();
        $order->load(array(
            'order_id' => $data ['order_id']
        ));
        //$orderinfo = $order->getOrderInformation();
        $hasSubscriptionProduct = $this->checkHasSubscriptionProductFromOrderID($data['order_id']);
        $currency = J2Store::currency();
        $currency_values = $this->getCurrency($order);
        $vars->currency_code = $currency_values['currency_code'];
        $vars->publish_key = $this->publish_key;
        // Set your secret key: remember to change this to your live secret key in production
        // See your keys here: https://dashboard.stripe.com/account/apikeys
        $this->setStripAppVersion();
        $rootURL = rtrim(JURI::base(),'/');
        $subpathURL = JURI::base(true);
        if(!empty($subpathURL) && ($subpathURL != '/')) {
            $rootURL = substr($rootURL, 0, -1 * strlen($subpathURL));
        }

        $success_url = $rootURL.JRoute::_('index.php?option=com_j2store&view=checkout&task=confirmPayment&orderpayment_type=' . $this->_element . '&paction=display', false);
        $cencel_url = $rootURL.JRoute::_('index.php?option=com_j2store&view=checkout&task=confirmPayment&orderpayment_type=' . $this->_element . '&paction=cancel', false);
        $config = J2Store::config();
        $items = $order->get_line_items($config->get('checkout_price_display_options', 1));
        $products = array();
        $itemamt = 0;
        foreach ($items as $item){
            $line_item = array();
            $desc = $item['name'];
            if (count($item['options'])) {
                foreach ($item['options'] as $option) {
                    $desc .= ' | ' .$option['name']. ':' .$option['value'];
                }
            }
            $desc = str_replace("'", '', $desc);
            $price = $currency->format ($item['amount'], $currency_values ['currency_code'], $currency_values ['currency_value'], $currency_values ['convert'] );
            $line_item['name'] = $desc;
            $line_item['description'] = $desc;
            $line_item['images'] = array();
            $line_item['amount'] = $price*100;
            $line_item['currency'] = strtolower($currency_values ['currency_code']);
            $line_item['quantity'] = $item['quantity'];
            $products[] = $line_item;
            $itemamt += $price*$item['quantity'];
        }
        //if excluding tax, then we should be sending the tax for the cart as an item as well
        if($config->get('checkout_price_display_options', 1) == 0 ) {
            if($order->order_tax > 0) {
                $tax = $currency->format ($order->order_tax, $currency_values ['currency_code'], $currency_values ['currency_value'], $currency_values ['convert'] );
                $line_item = array();
                $line_item['name'] = JText::_('J2STORE_CART_TAX');
                $line_item['description'] = JText::_('J2STORE_CART_TAX');
                $line_item['images'] = array();
                $line_item['amount'] = $tax*100;
                $line_item['currency'] = strtolower($currency_values ['currency_code']);
                $line_item['quantity'] = 1;
                $products[] = $line_item;
                $itemamt += $tax;
            }
        }

        //add discount
        /*$discount_amount_cart = $currency->format ( $order->get_total_discount($config->get('checkout_price_display_options', 1)), $currency_values ['currency_code'], $currency_values ['currency_value'], $currency_values ['convert'] );

        if ($discount_amount_cart > 0) {
            $line_item = array();
            $line_item['name'] = JText::_('J2STORE_CART_DISCOUNT');
            $line_item['description'] = JText::_('J2STORE_CART_DISCOUNT');
            $line_item['images'] = array();
            $line_item['amount'] = -$discount_amount_cart*100;
            $line_item['currency'] = strtolower($currency_values ['currency_code']);
            $line_item['quantity'] = 1;
            $products[] = $line_item;

            $itemamt = $discount_amount_cart;
        }*/

        $amount = $currency->format ( $order->order_total, $currency_values ['currency_code'], $currency_values ['currency_value'], $currency_values ['convert'] );
        if(round($amount,2) != round($itemamt,2)) {
            $description = JText::_ ( "J2STORE_STRIPE_ORDER_DESCRIPTION" ) . ": " . $order->order_id;
            //get invoice number
            $invoice_number = $order->getInvoiceNumber ();
            $products = array();
            $line_item = array();
            $line_item['name'] = $description;
            $line_item['description'] = $description;
            $line_item['images'] = array();
            $line_item['amount'] = $amount*100;
            $line_item['currency'] = strtolower($currency_values ['currency_code']);
            $line_item['quantity'] = 1;
            $products[] = $line_item;
        }
        $language = JFactory::getLanguage();
        $locale = substr($language->getTag(),0,2);
        $sripe_session_data = [
            'payment_method_types' => ['card'],
            'client_reference_id' => $order->order_id,
            'customer_email' => $order->user_email,
            'line_items' => $products,
            'success_url' => $success_url,
            'cancel_url' => $cencel_url,
            'locale' => $locale
        ];
        if($hasSubscriptionProduct){
            $sripe_session_data['mode'] = 'setup';
            unset($sripe_session_data['line_items']);
        }
        try{
            $session = \Stripe\Checkout\Session::create($sripe_session_data);
            if(isset($session->id) && !empty($session->id)){
                $vars->session_id = $session->id;
            }
        }catch (Exception $e){
            $vars->error = $e->getMessage();
        }

        $html = $this->_getLayout('prepayment', $vars);
        return $html;
    }

    /**
     * Processes the payment form
     * and returns HTML to be displayed to the user
     * generally with a success/failed message
     *
     * @param $data     array       form post data
     * @return string   HTML to display
     */
    function _postPayment($data)
    {
        // Process the payment
        $vars = new JObject ();

        $app = JFactory::getApplication();
        $paction = $app->input->getString('paction');

        switch ($paction) {
            case 'display' :
                $vars->message = JText::_($this->params->get('onafterpayment', ''));
                $html = $this->_getLayout('message', $vars);
                $html .= $this->_displayArticle();
                break;
            case 'cancel' :
                $vars->message = JText::_($this->params->get('oncancelpayment', ''));
                $html = $this->_getLayout('message', $vars);
                break;
            case 'process_intent_authentication':
                $result = $this->_process_intent_authentication();
                $json = json_encode($result);
                echo $json;
                $app->close();
                break;
            case 'process' :
                $result = $this->_process();
                $json = json_encode($result);
                echo $json;
                $app->close();
                break;
            default :
                $vars->message = JText::_($this->params->get('onerrorpayment', ''));
                $html = $this->_getLayout('message', $vars);
                break;
        }

        return $html;
    }

    function _process(){
        // You can find your endpoint's secret in your webhook settings
        $endpoint_secret = $this->secret_key;
        $payload = @file_get_contents('php://input');
        /*$sig_header = $_SERVER['HTTP_STRIPE_SIGNATURE'];
        $event = null;
        try {
            $this->setStripAppVersion();
            $event = \Stripe\Webhook::constructEvent(
                $payload, $sig_header, $endpoint_secret
            );
        } catch(\UnexpectedValueException $e) {
            // Invalid payload
            http_response_code(400);
            exit();
        } catch(\Stripe\Error\SignatureVerification $e) {
            // Invalid signature
            http_response_code(400);
            exit();
        }*/
        $event = json_decode($payload);
        // Handle the checkout.session.completed event
        if (isset($event->type) && $event->type == 'checkout.session.completed') {
            $session = $event->data->object;
            if(isset($session->client_reference_id) && !empty($session->client_reference_id)){
                $order = F0FTable::getInstance('Order', 'J2StoreTable')->getClone();
                $order->load(array(
                    'order_id' => $session->client_reference_id
                ));
                if(isset($order->order_id) && $order->order_id == $session->client_reference_id && !empty($session->client_reference_id) && $order->transaction_status != 'success'){
                    if(isset($session->mode) && $session->mode == 'setup'){
                        $this->processSubscriptionPayment($order, $session);
                    } else {
                        $order->transaction_id = $session->id;
                        $order->transaction_status = 'success';
                        $order->transaction_details = $this->_getFormattedTransactionDetails($session);
                        // change order status
                        $order->payment_complete();
                        $order->store();
                        $order->empty_cart();
                    }
                }
            }else{
                http_response_code(400);
                exit();
            }
        }else{
            http_response_code(400);
            exit();
        }
        echo 'OK';
        http_response_code(200);
        exit();
    }

    /**
     * Process subscription initial payment
     * */
    protected function processSubscriptionPayment($order, $session){
        $this->setStripAppVersion();
        $currency = J2Store::currency();
        $intent = \Stripe\SetupIntent::retrieve($session->setup_intent);
        if(!empty($intent)){
            $has_subscription = $this->checkHasSubscriptionProductFromOrderID($order->order_id);
            if($has_subscription){
                $payment_id = $intent->payment_method;
                $order_info = $order->getOrderInformation();
                $name = $order_info->billing_first_name . " " . $order_info->billing_last_name;
                $customer = \Stripe\Customer::create(array(
                    "name" => $name,
                    'payment_method' => $payment_id,
                    "email" => $order->user_email
                ));
                if(!empty($customer) && !empty($customer->id)){
                    $customer_id = $customer->id;
                    $currency_values = $this->getCurrency($order);
                    $amount = $currency->format($order->order_total, $currency_values ['currency_code'], $currency_values ['currency_value'], $currency_values ['convert']);
                    $check_currency = $this->checkCurrency($currency_values ['currency_code']);
                    if ($check_currency) {
                        $amount = round($amount, 0);
                    } else {
                        $amount = $amount * 100;
                    }

                    $invoice_number = $order->getInvoiceNumber();
                    $description = 'J2STORE_STRIPE_ORDER_DESCRIPTION_INITIAL_PAYMENT';
                    $description = JText::sprintf($description, $invoice_number);
                    $description = str_replace(array('[INVOICE_NUMBER]'), array($invoice_number), $description);
                    $card_update = false;
                    if ($order->order_type == 'subscription') {
                        $card_update = true;
                        $subscription = array();
                        $process = false;
                        $j2StorePlugin = J2Store::plugin();
                        $j2StorePlugin->event('ProcessSubscriptionCardUpdateCheckout', array($order->subscription_id, &$subscription, &$process));
                        if ($process) {
                            $this->updateStripeCustomerId($order->order_id, $customer_id, 1, $subscription);
                            $this->updateStripePaymentId($order->order_id, $payment_id, 1, $subscription);
                            $j2StorePlugin->event('CardUpdateSuccess', array($subscription));
                            // clear cart
                            $order->empty_cart();
                        }
                    } else {
                        $charge = array(
                            "amount" => $amount,
                            "currency" => $currency_values ['currency_code'],
                            "description" => $description,
                            "metadata" => array("invoice_number" => $invoice_number),
                            "customer" => $customer_id
                        );
                        $charge["payment_method"] = $payment_id;
                        $charge["off_session"] = "true";
                        $charge["payment_method_types"] = ['card'];
                        $charge["confirm"] = "true";

                        $model = F0FModel::getTmpInstance('AppPayment_stripecheckout', 'J2StoreModel');
                        try{
                            if($amount > 0){
                                $response = \Stripe\PaymentIntent::create($charge);
                                $this->_log($response, 'Payment Intent response');
                                $order->transaction_id = $response->id;
                                $order->transaction_details = $this->_getFormattedTransactionDetails($response);
                            }

                            $order->transaction_status = 'success';
                            if ($order->store()) {
                                $order->payment_complete();
                            }
                            $this->updateStripePaymentId($order->order_id, $payment_id);
                            $this->updateStripeCustomerId($order->order_id, $customer_id);
                        } catch (Exception $e){
                            $error = $e->getMessage();
                            $this->_log($error, 'Initial Payment Gateway Exception');
                            $error_body = $e->getJsonBody();
                            $this->_log(json_encode($error_body), 'Initial Payment Gateway Exception');
                            if(isset($error_body["error"]) && isset($error_body["error"]["decline_code"])){
                                if(!empty($error_body["error"]["decline_code"]) && $error_body["error"]["decline_code"] == "authentication_required"){
                                    if(!empty($error_body["error"]["payment_intent"]["client_secret"])){
                                        if(!empty($error_body["error"]["payment_method"]["id"])){
                                            $model->updateOrderMeta($order->j2store_order_id, 'stripe_client_secret', $error_body["error"]["payment_intent"]["client_secret"]);
                                            $model->updateOrderMeta($order->j2store_order_id, 'stripe_payment_method_id', $error_body["error"]["payment_method"]["id"]);
                                            $model->updateOrderMeta($order->j2store_order_id, 'stripe_payment_intent', $error_body["error"]["payment_intent"]["id"]);
                                            $order->add_history($error_body['error']['message']);
                                            $order->update_status(4);
                                            $this->updateStripePaymentAndCustomerId($order->order_id, $payment_id, $customer_id);
                                            $model->sendEmailToCustomerAboutAuthenticationRequiredForInitialPayment($order);
                                        }
                                    }
                                }
                            }
                        }
                    }
                } else {
                    $this->_log($customer, 'Create customer failed');
                }
            }
        } else {
            $this->_log($intent, 'Payment intent retrieve failed');
        }
    }

    /**
     * Get payment for renewal using billing ID
     * */
    protected function byRefference($subscription, $order)
    {
        $currency = J2Store::currency();
        $j2StorePlugin = J2Store::plugin();
        if (isset($subscription->meta['stripe_customer_id']['metavalue']) && $subscription->meta['stripe_customer_id']['metavalue'] != '') {
            if (isset($subscription->meta['stripe_payment_id']['metavalue']) && $subscription->meta['stripe_payment_id']['metavalue'] != '') {
                $payment_id = $subscription->meta['stripe_payment_id']['metavalue'];
                $customer_id = $subscription->meta['stripe_customer_id']['metavalue'];
                // initialise stripe object
                $this->setStripAppVersion();

                $currency_values = $this->getCurrency($order);
                $amount = $currency->format($order->order_total, $currency_values ['currency_code'], $currency_values ['currency_value'], $currency_values ['convert']);
                $check_currency = $this->checkCurrency($currency_values ['currency_code']);
                if ($check_currency) {
                    $amount = round($amount, 0);
                } else {
                    $amount = $amount * 100;
                }

                $invoice_number = $order->getInvoiceNumber();
                $description = $this->params->get('subscription_renewal_desc', 'J2STORE_STRIPE_CHECKOUT_ORDER_DESCRIPTION_RENEWAL_PAYMENT');
                if(empty($description)){
                    $description = 'J2STORE_STRIPE_CHECKOUT_ORDER_DESCRIPTION_RENEWAL_PAYMENT';
                }
                $description = JText::sprintf($description, $invoice_number);
                $product_name = '';
                $j2StorePlugin->event('GetSubscriptionProductName', array( &$product_name, $subscription->j2store_subscription_id ) );
                $description = str_replace(array('[INVOICE_NUMBER]', '[PRODUCT_NAME]'), array($invoice_number, $product_name), $description);

                $charge = array(
                    "amount" => $amount,
                    "currency" => $currency_values ['currency_code'],
                    "description" => $description,
                    "metadata" => array("invoice_number" => $invoice_number),
                    "customer" => $customer_id
                );
                $charge["payment_method"] = $payment_id;
                $charge["off_session"] = "true";
                $charge["payment_method_types"] = ['card'];
                $charge["confirm"] = "true";
                try{
                    $response = \Stripe\PaymentIntent::create($charge);
                    $this->_log($response, 'Payment Intent response');
                    $order->transaction_id = $response->id;
                    $order->transaction_status = 'success';
                    $order->transaction_details = $this->_getFormattedTransactionDetails($response);
                    $order->add_history('Response received from Stripe');
                    $transaction_id = $response->id;
                    $transaction_details = $order->transaction_details;
                    //$transaction_details = $this->_getFormattedTransactionDetails ( Stripe_Util::convertStripeObjectToArray ( $response ) );//Stripe_Util::convertStripeObjectToArray ( $response )
                    $this->_log($transaction_details, 'Payment Gateway Renewal Response');

                    $order->transaction_id = $transaction_id;
                    if ($order->store()) {
                        $j2StorePlugin->event('SuccessRenewalPayment', array($subscription, $order));
                    }
                    // change order status
                    $order->payment_complete();
                } catch (Exception $e){
                    $error = $e->getMessage();
                    $this->_log($error, 'Initial Payment Gateway Exception');
                    $error_body = $e->getJsonBody();
                    $this->_log(json_encode($error_body), 'Initial Payment Gateway Exception');
                    $model = F0FModel::getTmpInstance('AppPayment_stripecheckout', 'J2StoreModel');
                    if(isset($error_body["error"]) && isset($error_body["error"]["decline_code"])){
                        if(!empty($error_body["error"]["decline_code"]) && $error_body["error"]["decline_code"] == "authentication_required"){
                            if(!empty($error_body["error"]["payment_intent"]["client_secret"])){
                                if(!empty($error_body["error"]["payment_method"]["id"])){
                                    $model->updateOrderMeta($order->j2store_order_id, 'stripe_client_secret', $error_body["error"]["payment_intent"]["client_secret"]);
                                    $model->updateOrderMeta($order->j2store_order_id, 'stripe_payment_method_id', $error_body["error"]["payment_method"]["id"]);
                                    $model->updateOrderMeta($order->j2store_order_id, 'stripe_payment_intent', $error_body["error"]["payment_intent"]["id"]);
                                    $order->add_history($error_body['error']['message']);
                                    $order->update_status(4);
                                    $j2StorePlugin->event('AddSubscriptionHistory', array($subscription->j2store_subscription_id, $subscription->status, $error_body['error']['message']));
                                    $j2StorePlugin->event('PendingRenewalPayment', array($subscription, $order));
                                    $model->sendEmailToCustomerAboutAuthenticationRequiredForPayment($subscription, $order);
                                }
                            }
                        } else {
                            $retry_payment = true;
                            if (isset($error_body['error']) && isset($error_body['error']['decline_code'])) {
                                $decline_codes = array('lost_card', 'not_permitted', 'pickup_card', 'fraudulent', 'stolen_card');
                                if(in_array($error_body['error']['decline_code'], $decline_codes)){
                                    $retry_payment = false;
                                    $order->update_status(3);
                                    $message = sprintf(JText::_('J2STORE_STRIPE_RENEWALPAYMENT_FAILED_DUE_TO_CARD_ERROR'), $error_body['error']['decline_code']);
                                    $j2StorePlugin->event('AddSubscriptionHistory', array($subscription->j2store_subscription_id, $subscription->status, $message));
                                    $j2StorePlugin->event('UpdateCardExpiredForSubscription', array($subscription, $order));
                                }
                            }
                            if($retry_payment === true){
                                if (isset($error_body['error']) && isset($error_body['error']['type']) && isset($error_body['error']['param'])) {
                                    if ($error_body['error']['type'] == 'invalid_request_error' && $error_body['error']['param'] == 'customer') {
                                        $j2StorePlugin->event('NoResponseForRenewalPayment', array($subscription, $order));
                                    } elseif ($error_body['error']['type'] == 'card_error' && $error_body['error']['param'] == 'card') {
                                        $j2StorePlugin->event('NoResponseForRenewalPayment', array($subscription, $order));
                                    } else {
                                        if(isset($error_body['error']['message'])){
                                            $j2StorePlugin->event('AddSubscriptionHistory', array($subscription->j2store_subscription_id, $subscription->status, $error_body['error']['message']));
                                        }
                                        $j2StorePlugin->event('NoResponseForRenewalPayment', array($subscription, $order));
                                    }
                                } else if(isset($error_body['error'])){
                                    if(isset($error_body['error']['message'])){
                                        $j2StorePlugin->event('AddSubscriptionHistory', array($subscription->j2store_subscription_id, $subscription->status, $error_body['error']['message']));
                                    }
                                    $j2StorePlugin->event('NoResponseForRenewalPayment', array($subscription, $order));
                                } else {
                                    $j2StorePlugin->event('NoResponseForRenewalPayment', array($subscription, $order));
                                }
                            }
                        }
                    } else {
                        if (isset($error_body['error']) && isset($error_body['error']['type']) && isset($error_body['error']['param'])) {
                            if ($error_body['error']['type'] == 'invalid_request_error' && $error_body['error']['param'] == 'customer') {
                                $j2StorePlugin->event('NoResponseForRenewalPayment', array($subscription, $order));
                            } elseif ($error_body['error']['type'] == 'card_error' && $error_body['error']['param'] == 'card') {
                                $j2StorePlugin->event('NoResponseForRenewalPayment', array($subscription, $order));
                            } else {
                                if(isset($error_body['error']['message'])){
                                    $j2StorePlugin->event('AddSubscriptionHistory', array($subscription->j2store_subscription_id, $subscription->status, $error_body['error']['message']));
                                }
                                $j2StorePlugin->event('NoResponseForRenewalPayment', array($subscription, $order));
                            }
                        } else if(isset($error_body['error'])){
                            if(isset($error_body['error']['message'])){
                                $j2StorePlugin->event('AddSubscriptionHistory', array($subscription->j2store_subscription_id, $subscription->status, $error_body['error']['message']));
                            }
                            $j2StorePlugin->event('NoResponseForRenewalPayment', array($subscription, $order));
                        } else {
                            $j2StorePlugin->event('NoResponseForRenewalPayment', array($subscription, $order));
                        }
                    }
                }
            } else {
                $order->update_status(3);
                $j2StorePlugin->event('FailedRenewalPayment', array($subscription, $order));
                $returnVal = "Payment failed due to empty payment ID";
                $this->_log($returnVal);
            }
        } else {
            $order->update_status(3);
            $j2StorePlugin->event('FailedRenewalPayment', array($subscription, $order));
            $returnVal = "Payment failed due to empty customer ID";
            $this->_log($returnVal);
        }
    }

    /**
     * Authenticate the pending payment
     * */
    protected function _process_intent_authentication(){
        $app = JFactory::getApplication();
        $json = array();
        // get the post data
        $data = $app->input->getArray($_POST);
        if (!JRequest::checkToken()) {
            $json ['error'] = JText::_('J2STORE_STRIPE_INVALID_TOKEN');
        }
        // run initial checks
        if (!isset($data ['order_id']) || empty ($data ['order_id'])) {
            $json ['error'] = JText::_('J2STORE_STRIPE_INVALID_ORDER');
        }

        if(!$json && isset($data ['order_id']) && !empty($data ['order_id']) ){
            // get order information from table
            F0FTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_j2store/tables');
            $order = F0FTable::getInstance('Order', 'J2StoreTable')->getClone();
            $order->load(array(
                'order_id' => $data ['order_id']
            ));
            if(!empty($order) && !empty($order->j2store_order_id)){
                $model = F0FModel::getTmpInstance('AppPayment_stripecheckout', 'J2StoreModel');
                $payment_intent_id = $model->getMetaData($order->j2store_order_id, 'stripe_payment_intent', 'order');
                if(!empty($payment_intent_id)){
                    try{
                        // See your keys here: https://dashboard.stripe.com/account/apikeys
                        $this->setStripAppVersion();
                        $intent = \Stripe\PaymentIntent::retrieve($payment_intent_id);
                        $this->_log(json_encode($intent), 'Payment Gateway Response');
                        if(isset($intent->id) && !empty($intent->id)){
                            $order->transaction_id = $intent->id;
                            $order->transaction_status = $intent->status;
                            $subscription = '';
                            $j2StorePlugin = J2Store::plugin();
                            if($order->subscription_id){
                                $subscription = $model->getSubscriptionById($order->subscription_id);
                            }
                            if (isset ($intent->status) && $intent->status == 'succeeded') {
                                // change order status
                                $order->payment_complete();
                                if(!empty($subscription)){
                                    $j2StorePlugin->event('SuccessRenewalPayment', array($subscription, $order, false));
                                } else {
                                    $this->markSubscriptionActive($order->order_id);
                                }
                                // remove items from cart
                                // clear cart
                                $order->empty_cart();
                            } else if(isset ($intent->status) && $intent->status == 'requires_source'){
                                $order->add_history($intent->last_payment_error);
                                $json ['error'] = empty($intent->last_payment_error) ? JText::_($this->params->get('onerrorpayment', '')): $intent->last_payment_error;
                            } else {
                                // order failed.
                                /**
                                 * Note: We are not marking the status as failed because the customer cannot retry the same order again if we mark as failed
                                 * Also we have changed the renewal date of subscription while thr payment marked as pending.
                                 * The site owner might unaware of renewal date update, in case if he marked the subscription as active to retry the renewal.
                                 * */
                                /*$order->update_status(3);
                                if(!empty($subscription)){
                                    $j2StorePlugin->event('FailedRenewalPayment', array($subscription, $order));
                                }*/
                                $order->add_history($intent->last_payment_error);
                                $json ['error'] = empty($intent->last_payment_error) ? JText::_($this->params->get('onerrorpayment', '')): $intent->last_payment_error;
                            }

                            $order->transaction_details = json_encode($intent);
                            if ($order->store()) {
                                if (isset ($order->order_state_id) && $order->order_state_id == 1) {
                                    $json ['success'] = JText::_($this->params->get('onafterpayment', ''));
                                    $json ['redirect'] = JRoute::_('index.php?option=com_j2store&view=checkout&task=confirmPayment&orderpayment_type=' . $this->_element . '&paction=display',false);
                                }
                            } else {
                                $json ['error'] = JText::_('J2STORE_STRIPE_ERROR_UPDATING_ORDER');
                            }
                        }else{
                            $json ['error'] = JText::_('J2STORE_STRIPE_INVALID_PAYMENT_INTENT_ID');
                        }
                    }catch (Exception $e){
                        $json ['error'] = $e->getMessage();
                    }
                } else {
                    $json ['error'] = JText::_('J2STORE_STRIPE_PAYMENT_INTENT_ID_NOT_FOUND');
                }
            } else {
                $json ['error'] = JText::_('J2STORE_STRIPE_INVALID_ORDER');
            }
        }

        return $json;
    }


    /**
     * Simple logger
     *
     * @param string $text
     * @param string $type
     * @return void
     */
    function _log($text, $type = 'message')
    {
        if ($this->_isLog) {
            $file = JPATH_ROOT . "/cache/{$this->_element}.log";
            $date = JFactory::getDate();

            $f = fopen($file, 'a');
            fwrite($f, "\n\n" . $date->format('Y-m-d H:i:s'));
            fwrite($f, "\n" . $type . ': ' . $text);
            fclose($f);
        }
    }

    function checkCurrency($code)
    {
        $zero_decimal_currency = array(
            'BIF',
            'CLP',
            'DJF',
            'GNF',
            'JPY',
            'KMF',
            'KRW',
            'MGA',
            'PYG',
            'RWF',
            'VND',
            'VUV',
            'XAF',
            'XOF',
            'XPF'
        );

        if (in_array($code, $zero_decimal_currency)) {
            return true;
        }
        return false;
    }

    // change array of data to string
    function _getFormattedTransactionDetails($data)
    {
        return json_encode($data);
    }

    /**
     * Update Stripe customer Id
     * */
    protected function updateStripeCustomerId($order_id, $customerID, $updateCard = 0, $subscription = array())
    {
        if ($updateCard) {
            $subscriptions[] = $subscription;
        } else {
            $subscriptions = $this->getSubscriptionByOrderId($order_id);
        }

        $j2StorePlugin = J2Store::plugin();
        if (is_array($subscriptions) && count($subscriptions)) {
            foreach ($subscriptions as $susb) {
                if ($updateCard) {
                    $comment = JText::_('J2STORE_SUBSCRIPTION_HISTORY_PAYMENT_COMPLETED_FOR_UPDATE_CARD');
                } else {
                    $comment = JText::_('J2STORE_SUBSCRIPTION_HISTORY_PAYMENT_COMPLETED');
                }

                $j2StorePlugin->event('AddSubscriptionHistory', array($susb->j2store_subscription_id, $susb->status, $comment));
                $j2StorePlugin->event('ChangeSubscriptionStatus', array($susb->j2store_subscription_id, 'active'));
                $j2StorePlugin->event('RefreshUserGroups', array($susb->user_id));

                $this->addSubscriptionMeta($susb->j2store_subscription_id, 'stripe_customer_id', $customerID);
            }
        }

    }

    protected function markSubscriptionActive($order_id){
        $subscriptions = $this->getSubscriptionByOrderId($order_id);
        $j2StorePlugin = J2Store::plugin();
        if (is_array($subscriptions) && count($subscriptions)) {
            foreach ($subscriptions as $susb) {
                $comment = JText::_('J2STORE_SUBSCRIPTION_HISTORY_PAYMENT_COMPLETED');

                $j2StorePlugin->event('AddSubscriptionHistory', array($susb->j2store_subscription_id, $susb->status, $comment));
                $j2StorePlugin->event('ChangeSubscriptionStatus', array($susb->j2store_subscription_id, 'active'));
                $j2StorePlugin->event('RefreshUserGroups', array($susb->user_id));
            }
        }
    }

    /**
     * Update Stripe customer Id
     * */
    protected function updateStripePaymentId($order_id, $paymentID, $updateCard = 0, $subscription = array())
    {
        if ($updateCard) {
            $subscriptions[] = $subscription;
        } else {
            $subscriptions = $this->getSubscriptionByOrderId($order_id);
        }

        $j2StorePlugin = J2Store::plugin();
        if (is_array($subscriptions) && count($subscriptions)) {
            foreach ($subscriptions as $susb) {
                $this->addSubscriptionMeta($susb->j2store_subscription_id, 'stripe_payment_id', $paymentID);
            }
        }

    }

    /**
     * Update Stripe customer Id
     * */
    protected function updateStripePaymentAndCustomerId($order_id, $paymentID, $customerID)
    {
        $subscriptions = $this->getSubscriptionByOrderId($order_id);
        if (is_array($subscriptions) && count($subscriptions)) {
            foreach ($subscriptions as $susb) {
                $this->addSubscriptionMeta($susb->j2store_subscription_id, 'stripe_payment_id', $paymentID);
                $this->addSubscriptionMeta($susb->j2store_subscription_id, 'stripe_customer_id', $customerID);
            }
        }

    }

    /**
     * To update subscription meta
     * */
    protected function addSubscriptionMeta($subscription_id, $key, $value)
    {
        $j2StorePlugin = J2Store::plugin();
        $j2StorePlugin->event('AddSubscriptionMeta', array($subscription_id, $key, $value));
    }

    /**
     * Get subscriptions based on order id
     * */
    protected function getSubscriptionByOrderId($order_id)
    {
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);
        $query->select('*');
        $query->from('#__j2store_subscriptions');
        $query->where('order_id = ' . $db->quote($order_id));

        $db->setQuery($query);
        $result = $db->loadObjectList();

        return $result;
    }
}